@@ -11,7 +11,7 @@ import concurrent.futures as thd
 from matplotlib import gridspec as grd
 
 
-# Funtrion that perform a scan of a given data histogram and compare it to a reference background histogram.
+# Function that performs a scan of a given data histogram and compares it to a reference background histogram.
 # This function is used by the BumpHunter class methods and is not intended to be used directly.
 def scan_hist(hist,ref,w_ar,self,ih):
             '''
@@ -51,7 +51,8 @@ def scan_hist(hist,ref,w_ar,self,ih):
             
             # Create the results array
             res = np.empty(w_ar.size,dtype=np.object)
-            min_Pval,min_loc = np.empty(w_ar.size),np.empty(w_ar.size)
+            min_Pval,min_loc = np.empty(w_ar.size),np.empty(w_ar.size,dtype=int)
+            signal_eval = np.empty(w_ar.size)
             
             # Loop over all the width of the window
             i = 0
@@ -67,6 +68,16 @@ def scan_hist(hist,ref,w_ar,self,ih):
                 # Define possition range
                 pos = np.arange(Hinf,Hsup-w+1,scan_stepp)
                 
+                # Check that there is at least one interval to check for width w
+                # If not, we must set dummy values in order to avoid crashes
+                if(pos.size==0):
+                    res[i] = np.array([1.0])
+                    min_Pval[i] = 1.0
+                    min_loc[i] = 0
+                    signal_eval[i] = 0
+                    i+=1
+                    continue
+                
                 # Initialize local p-value array for width w
                 res[i] = np.empty(pos.size)
                 
@@ -80,20 +91,25 @@ def scan_hist(hist,ref,w_ar,self,ih):
                     res[i][Nhist<=Nref] = 1.0
                     res[i][Nhist>Nref] = G(Nhist[Nhist>Nref],Nref[Nhist>Nref])
                 elif(self.mode=='deficit'):
-                    res[i][Nhist<=Nref] = 1.0-G(Nhist[Nhist<=Nref]+1,Nref[Nhist<=Nref])
-                    res[i][Nhist>Nref] = 1.0
-                res[i][(Nhist==0) & (Nref==0)] = 1.0
+                    res[i][Nhist<Nref] = 1.0-G(Nhist[Nhist<Nref]+1,Nref[Nhist<Nref])
+                    res[i][Nhist>=Nref] = 1.0
                 res[i][(Nref==0) & (Nhist>0)] = 1.0 # To be consistant with c++ results
                 
                 # Get the minimum p-value and associated position for width w
                 min_Pval[i] = res[i].min()
                 min_loc[i] = pos[res[i].argmin()]
-    
+                signal_eval[i] = Nhist[min_loc[i]] - Nref[min_loc[i]]
+                
                 i += 1
             
-            # Get the minimum p-value and associated windonw among all width
+            # Get the minimum p-value and associated window among all width
             min_width = w_ar[min_Pval.argmin()]
             min_loc = min_loc[min_Pval.argmin()]
+            
+            # Evaluate the number of signal event (for data only)
+            if(ih==0):
+                self.signal_eval = signal_eval[min_Pval.argmin()]
+            
             min_Pval = min_Pval.min()
             
             # Save the results in inner variables and return
@@ -101,7 +117,7 @@ def scan_hist(hist,ref,w_ar,self,ih):
             self.min_Pval_ar[ih] = min_Pval
             self.min_loc_ar[ih] = int(min_loc)
             self.min_width_ar[ih] = int(min_width)
-            return 
+            return
 
 
 # THE super BumpHunter class
@@ -109,11 +125,11 @@ class BumpHunter():
     '''
     The BumpHunter class is the object providing all the necessary tools to "bump hunt" with ease.
     
-    It comes with a set of methos to perform scans using the BumpHunter algorithm and to manage all
+    It comes with a set of methods to perform scans using the BumpHunter algorithm and to manage all
     the parameters and results stored in the inner variables.
     
     List of inner parameter variables :
-        rang : x-axis range of the histograms. Also define the range in which the scan wiil be performed.
+        rang : x-axis range of the histograms. Also define the range in which the scan will be performed.
         
         mode : String specifying if the algorithm must look for a excess or a deficit in the data.
                Can be either 'excess' or 'deficit'. Default to 'excess'.
@@ -123,7 +139,7 @@ class BumpHunter():
         
         width_max : Maximum value of the scan window width that should be tested (in number of bins). Can be
                     either None or a positive integer. if None, the value is set to the total number of bins
-                    of the histograms ivided by 2. Default to none.
+                    of the histograms divided by 2. Default to none.
         
         width_step : Number of bins by which the scan window width is increased at each step. Default to 1.
         
@@ -143,11 +159,11 @@ class BumpHunter():
                Default to 60.
         
         weights : Weights for the background distribution. Can be either None or a array-like of float.
-                  If array-like of floats, each background events will be acounted by its weights when making
+                  If array-like of floats, each background events will be accounted by its weights when making
                   histograms. The size of the array-like must be the same than of bkg. The same weights are
                   considered when sampling the pseudo-data.
                   If None, no weights will be considered.
-                  Default to 1.
+                  Default to None.
         
         Nworker : Number of thread to be run in parallel when scanning all the histograms (data and pseudo-data).
                   If less or equal to 1, then parallelism will be disabled.
@@ -161,7 +177,7 @@ class BumpHunter():
         
         str_step : Increase of the signal stregth to be injected in the background at each iteration. Default to 0.25.
         
-        str_scale : Specify how the signal strength should vary. If 'log', the the signal strength will vary according to
+        str_scale : Specify how the signal strength should vary. If 'log', the signal strength will vary according to
                     a log scale starting from 10**Str_min. If 'lin', the signal will vary according to a linear scale starting
                     from Str_min with a step of Str_step.
                     Default to 'lin'.
@@ -196,8 +212,10 @@ class BumpHunter():
         min_width_ar : Array containing the width of the windows for which the minimum p-value has been found for
                        the data (indice=0) and pseudo-data (indice>0).
         
+        signal_eval : Number of signal events evaluated form the last scan.
+        
         signal_min : Minimum number of signal events ones must inject in the data in order to reach the required
-                     significance.
+                     significance (might differ from signal_eval).
         
         signal_ratio : Ratio signal_min/signal_exp (signal strength).
         
@@ -251,6 +269,7 @@ class BumpHunter():
         self.min_loc_ar = []
         self.min_width_ar = []
         self.t_ar = []
+        self.signal_eval = 0
         self.signal_min = 0
         self.signal_ratio = None
         self.data_inject = []
@@ -465,6 +484,8 @@ class BumpHunter():
             
             min_width_ar : Array containing the width of the windows for which the minimum p-value has been found for
                            the data (indice=0) and pseudo-data (indice>0).
+            
+            signal_eval : Number of signal events evaluated form the last scan.
         '''
         
         # Set the seed if required (or reset it if None)
@@ -544,7 +565,7 @@ class BumpHunter():
         reach a target significance. This function use the BumpHunter algorithm in order to calculate the reached
         significance for a given signal strength.
         
-        This function share most of its parameters with the BumpHunter function.
+        This method share most of its parameters with the BumpScan method.
         
         Arguments :
             sig : Numpy array containing the simulated signal. This distribution will be used to perform the signal
@@ -565,7 +586,7 @@ class BumpHunter():
             
             sigma_ar : Numpy array containing the significance values obtained at each step.
         
-        All the result global variables of the BumpHunter function will be filled with the results of the scan permormed
+        All the result inner variables of the BumpHunter instance will be filled with the results of the scan permormed
         during the last iteration (when sigma_limit is reached).
         '''
         
@@ -756,7 +777,7 @@ class BumpHunter():
         print('   Signal strength : {0:1.4f}'.format(self.signal_ratio))
         print('')
         
-        # Save the data obtained after last injection in inner variable
+        # Save the data obtained after last injection in inner variables
         self.data_inject = data_hist
         
         # Append the last step results to the background results
@@ -780,7 +801,7 @@ class BumpHunter():
         window.
         
         Arguments :
-            data : Numpy array containing the raw unbined data.
+            data : Numpy array containing the data.
             
             is_hist : Boolean specifying if data is in histogram form or not. Default to False.
         
@@ -834,9 +855,9 @@ class BumpHunter():
         Plot the data and bakground histograms with the bump found by BumpHunter highlighted.
         
         Arguments :
-            data : Numpy array containing the raw unbined data.
+            data : Numpy array containing the data.
             
-            bkg : Numpy array containing the raw unbined background.
+            bkg : Numpy array containing the background.
             
             is_hist : Boolean specifying if data and bkg are in histogram form or not. Default to False.
             
@@ -865,13 +886,12 @@ class BumpHunter():
         
         # Calculate significance for each bin
         sig = np.empty(Hbkg.size)
-        sig[(H[0]==0) & (Hbkg==0)]=1.0
-        sig[H[0]>=Hbkg] = G(H[0][H[0]>=Hbkg],Hbkg[H[0]>=Hbkg])
+        sig[H[0]>Hbkg] = G(H[0][H[0]>Hbkg],Hbkg[H[0]>Hbkg])
         sig[H[0]<Hbkg] = 1-G(H[0][H[0]<Hbkg]+1,Hbkg[H[0]<Hbkg])
+        sig[H[0]==Hbkg] = 1.0
         sig = norm.ppf(1-sig)
         sig[sig<0] = 0 # If negative, set it to 0
-        sig[sig==np.inf] = 0 # Avoid errors
-        sig[sig==np.NaN] = 0
+        np.nan_to_num(sig, posinf=0, neginf=0, nan=0, copy=False) # Avoid errors
         sig[H[0]<Hbkg] = -sig[H[0]<Hbkg]  # Now we can make it signed
         
         # Plot the test histograms with the bump found by BumpHunter plus a little significance plot
@@ -893,7 +913,7 @@ class BumpHunter():
                          ls='',color='blue',label='data')
         
         plt.plot(np.full(2,Bmin),np.array([0,H[0][self.min_loc_ar[0]]]),'r--',label=('BUMP'))
-        plt.plot(np.full(2,Bmax),np.array([0,H[0][self.min_loc_ar[0]+self.min_width_ar[0]]]),'r--')
+        plt.plot(np.full(2,Bmax),np.array([0,H[0][self.min_loc_ar[0]+self.min_width_ar[0]-1]]),'r--')
         plt.legend(fontsize='large')
         plt.yscale('log')
         if self.rang!=None:
@@ -1036,9 +1056,9 @@ class BumpHunter():
         Information are printed to stdout.
         
         Arguments :
-            data : Numpy array containing the raw unbined data.
+            data : Numpy array containing the data.
             
-            bkg : Numpy array containing the raw unbined background.
+            bkg : Numpy array containing the background.
             
             is_hist : Boolean specifying if data and bkg are in histogram form or not. Default to False.
         '''
@@ -1057,14 +1077,12 @@ class BumpHunter():
         Bmax = H[1][self.min_loc_ar[0]+self.min_width_ar[0]]
         Bmean = (Bmax+Bmin)/2
         Bwidth = Bmax-Bmin
-        D =  H[0][self.min_loc_ar[0]:self.min_loc_ar[0]+self.min_width_ar[0]].sum()
-        B = Hb[self.min_loc_ar[0]:self.min_loc_ar[0]+self.min_width_ar[0]].sum()
         
         print('   min : {0:.3f}'.format(Bmin))
         print('   max : {0:.3f}'.format(Bmax))
         print('   mean : {0:.3f}'.format(Bmean))
         print('   width : {0:.3f}'.format(Bwidth))
-        print('   number of signal events : {}'.format(D-B))
+        print('   number of signal events : {}'.format(self.signal_eval))
         print('   global p-value : {0:1.5f}'.format(self.global_Pval))
         print('   significance = {0:1.5f}'.format(self.significance))
         print('')
